from .construct_file import frame_constructor
from .dbserver.data_manager import data_manager
db_config = None


def get_db_index(db_config):
    return db_config['user'] + '@' + db_config['host'] + ':' + db_config['database']

class Data(object):
    Base_pool = {}
    try:
        exec('from config import db_config')
        db_config = getattr(db_config, "db_config")
        # print(db_config)
        if isinstance(db_config,dict):
            Base = data_manager(db_config)
            Base_pool['default'] = Base
            db_index = get_db_index(db_config)
            Base_pool[db_index] = Base
        print('Auto init success!')
    except:
        print('Did not find a db config file, need run Data.init(db_config) manually...')
        # Base = None


    @staticmethod
    def init(db_config):
        Base = data_manager(db_config)
        if len(Data.Base_pool.keys()) == 0:
            Data.Base_pool['default'] = Base
        db_index = get_db_index(db_config)
        if db_index not in Data.Base_pool:
            Data.Base_pool[db_index] = Base

    @staticmethod
    def check_connections(base_id='default',show_manager_id=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')
        if base_id not in Data.Base_pool:
            return
        sql_pool = Data.Base_pool[base_id].sql_pool
        busy_list = []
        all_list = []
        free_list = []
        sql_executing = {}
        for sql_id in sql_pool:
            sql = sql_pool[sql_id]
            if sql.is_busy() is True:
                busy_list.append(sql_id)
                sql_executing[sql_id] = [sql.executing_query,sql.last_execute_time]


            if sql.is_busy() is False:
                free_list.append(sql_id)
            all_list.append(sql_id)

        result = {
            'busy_base': {
                'count': len(busy_list),
                'lists': busy_list,
                'executing': sql_executing,
            },
            'free_base': {
                'count': len(free_list),
                'lists': free_list
            },
            'all_base': {
                'count': len(all_list),
                'lists': all_list
            }
        }
        return result

    @staticmethod
    def add_new_sql(base_id='default',show_manager_id=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')

        return Data.Base_pool[base_id].new() if base_id in Data.Base_pool else None

    @staticmethod
    def create(table, colums, comment='',show_sql=False,base_id='default',show_manager_id=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')
        return Data.Base_pool[base_id].create(table, colums, comment,show_sql) if base_id in Data.Base_pool else None

    @staticmethod
    def insert(table, params,show_sql=False,base_id='default',show_manager_id=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')

        data = Data.Base_pool[base_id].insert(table, params, show_sql) if base_id in Data.Base_pool else None
        return data

    @staticmethod
    def find(table, conditions, fields=('*',), or_cond=None, order=None, show_sql=False,base_id='default',show_manager_id=False,from_cache=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')

        return Data.Base_pool[base_id].find(table, conditions, or_cond, fields, order, show_sql,from_cache) if base_id in Data.Base_pool else None

    @staticmethod
    def select(table, conditions, fields=('*',),or_cond=None, group=None, order=None, limit=None, show_sql=False,base_id='default',show_manager_id=False,from_cache=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')
        return Data.Base_pool[base_id].select(table, conditions, or_cond, fields, group, order, limit, show_sql,from_cache) if base_id in Data.Base_pool else None


    @staticmethod
    def find_last(table, conditions, info='id', limit=1, fields=("*",), show_sql=False):
        res = Data.select(table, conditions, fields=fields, limit=[limit], order = (info,'desc'),show_sql=show_sql)
        if res == None:
            return
        else:
            return res[0]
    # 这个函数不应该使用

    @staticmethod
    def update(table, conditions, params=None, or_cond=None,show_sql=False,base_id='default',show_manager_id=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')
        # print(params)
        Data.Base_pool[base_id].update(table, conditions,or_cond, params,show_sql) if base_id in Data.Base_pool else None
        return

    @staticmethod
    def delete(table, conditions, or_cond=None, show_sql=False,base_id='default',show_manager_id=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')
        data = Data.Base_pool[base_id].delete(table, conditions, or_cond, show_sql) if base_id in Data.Base_pool else None
        return data

    @staticmethod
    def truncate(table, show_sql=False, base_id='default', show_manager_id=False):
        if show_manager_id is True:
            print('本次任务通过', base_id, '执行')
        return Data.Base_pool[base_id].truncate(table, show_sql) if base_id in Data.Base_pool else None


    @staticmethod
    def query(sql, show_sql=False,base_id='default',show_manager_id=False):
        if show_manager_id is True:
            print('本次任务通过',base_id,'执行')
        return Data.Base_pool[base_id].query(sql, show_sql) if base_id in Data.Base_pool else None

    @staticmethod
    def get_cache(table,safe=True):
        if safe:
            pass
        res = {}
        all_data = Data.select(table,[])
        for i in all_data:
            res[i['id']] = i
        return res

    @staticmethod
    def map_all_db(base_id='default',show_manager_id=False,file_path='',file_name='db_frame.py'):
        try:
            if show_manager_id is True:
                print('本次任务通过', base_id, '执行')
            if base_id not in Data.Base_pool:
                print('base id not exist.. construct fail')
                return
            base_manager = Data.Base_pool[base_id]
            db_name = base_manager.t_data['database']
            all_table = Data.query('show tables',base_id=base_id)
            table_index = {}
            for i in all_table:
                table_name = i[0]
                table_info = Data.find('information_schema.tables',[('table_schema','=',db_name),('table_name','=',table_name)],fields=('table_name','table_comment'))
                res = Data.query('show full columns from '+str(table_name),base_id=base_id)
                table_index[table_name] = list(res)
                table_index[table_name].append(table_info)
            # pprint(table_index)
            constructor = frame_constructor(db_name,table_index,file_path,file_name)
            constructor.dump()
            print('create file success in %s'%file_path+file_name)
            return table_index
        except Exception as e:
            print(str(e))













_indent = '    '
_change_line = '\n'
_func_define = 'def '
_func_return = 'return'


class frame_constructor(object):
    def __init__(self,db_name,table_struct,file_path,file_name):
        self.table_struct = table_struct
        self.db_name = db_name
        self.file_path = file_path
        self.file_name = file_name
        self.fh = open(self.file_path+self.file_name,'w')
        self.func_name_list = []


    def dump(self):
        self.add_file_title()
        for table_name,table_frame in self.table_struct.items():
            self.construct_table(table_name,table_frame)
        self.write_main_run()

        self.fh.close()

    def add_file_title(self):
        self.write_comment(' db-name = '+self.db_name,indent = 0)
        self.changeline()
        self.fh.write('from anduin.server import Data')
        self.changeline()
        self.changeline()

    def construct_table(self,table_name,table_frame):
        self.construct_func_def(table_name)
        self.func_name_list.append(table_name)
        self.changeline()
        self.indent()
        self.write_drop_table(table_name)
        self.changeline()
        self.indent()
        self.write_table_frame(table_frame)
        self.write_create_table(table_frame)
        self.changeline()
        self.indent()
        self.add_return()
        self.changeline(2)



    def construct_func_def(self,func_name):
        self.fh.write(_func_define + func_name + '():')

    def add_return(self):
        self.fh.write(_func_return)

    def indent(self,indent_multi = 1):
        self.fh.write(_indent*indent_multi)

    def changeline(self,change_mult = 1):
        self.fh.write(_change_line*change_mult)

    def write_comment(self,content,indent = 2):
        if content != None:
            for i in content.split('\n'):
                self.indent(indent)
                self.fh.write('#'+i)

    def write_drop_table(self,table_name):
        self.fh.write('table_name = "%s"'%(table_name))
        self.changeline()
        self.indent()
        self.fh.write('Data.query("drop table %s"%table_name)')

    def write_table_frame(self, table_frame):
        self.fh.write('column = [')
        self.changeline()
        # print(table_frame)
        for line in table_frame:
            if isinstance(line,dict):
                continue
            self.indent(2)
            self.fh.write('(')
            # for word in line:
            if 'PRI' in line:
                self.fh.write('"%s", "int", '%(line[0]))
            else:
                self.fh.write('"%s", "%s", ' % (line[0], line[1]))
            if line[5] != None:
                self.fh.write('"default \'%s\'", '%line[5])
            if 'PRI' in line:
                self.fh.write('"%s", "%s", '%('AUTO_INCREMENT','primary key'))
            if line[8] != None:
                self.fh.write('"comment \'%s\'" ,' % line[8])
            self.fh.write('),')
            self.changeline()
            self.write_comment(line[8])
            self.changeline()
        self.indent()
        self.fh.write(']')
        self.changeline()

    def write_create_table(self,table_frame):
        self.indent()
        self.fh.write('Data.create(table_name, column, comment="%s")'%table_frame[-1]['table_comment'])

    def write_main_run(self):
        self.construct_func_def('create_all_table')
        self.changeline()
        for func_name in self.func_name_list:
            self.indent()
            self.fh.write(func_name+'()')
            self.changeline()
        pass











import threading
import time
# import _thread as thread

TIME_ACCURACY = 1  # 时间精度，时间精度不是越小越好！你的Task每次循环超过了这个值,将影响准确度


def IntervalTask(sec, func, params=(), immediatly=True, thread_name=''):
    def run(*params):
        if immediatly is False:
            time.sleep(sec)
        while 1:
            func(*params)
            time.sleep(sec)

    # print(params)
    t = threading.Thread(target=run, args=params)
    if thread_name != '':
        t.name = thread_name
    t.start()
    # t.join()


def normal_task(sec, func, params=(), thread_name=''):
    def run(*params):
        time.sleep(sec)
        func(*params)

    t = threading.Thread(target=run, args=params)
    if thread_name != '':
        t.name = thread_name
    t.start()
    # t.join()
    # threading.enumerate()
    # print(threading.enumerate())



if "__main__" == __name__:
    # res = [(1,2,3),(4,5,6),(7,8,9)]
    # print(turbo(foo,res))
    pass



import codecs
import os
from setuptools import setup

def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    # 以下为必需参数
    name='anduin',  # 模块名
    version='1.6.7',  # 当前版本
    description='a lite mysql & sqlite3 connect engine...',  # 简短描述
    py_modules=["anduin"],  # 单文件模块写法
    # ckages=find_packages(exclude=['contrib', 'docs', 'tests']),  # 多文件模块写法
    license='MIT',
    long_description=read("README.rst"),
    author='campanula',
    author_email='421248329@qq.com',
    platforms='any',
    keywords="mysql , sqlite3 , sql engine",
    # packages = find_packages('anduin'),
    # package_dir = {'anduin':'*'},
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3', 
    ],
    url='https://github.com/campanulamediuml/Anduin',
    install_requires=[
        'pymysql==0.10.1',
    ],
    include_package_data=True,
    zip_safe=True,
    packages=['anduin', 'anduin/dbserver'],
    python_requires='>=3.2',

)



import time
from ..Scheduler import IntervalTask
from ..dbserver.base import Base, ENGINE_DICT, mysql


# from config.config import db_config


def dump_table(table, sql):
    if sql.is_busy():
        return
    sql.become_busy()
    result = sql.select(table, [])
    sql.become_free()
    return result

class data_manager(object):
    min_keep_connection = 10
    keep_cycle = 10

    def __init__(self, db_config):
        self.t_data = db_config
        if 'engine' not in self.t_data or self.t_data['engine'] not in ENGINE_DICT:
            self.t_data['engine'] = mysql

        self.sql_pool = {}
        print('creating DB connection pool...')
        self.new()
        # print('connect done!')
        IntervalTask(data_manager.keep_cycle, self.keep_connect)

        self.mem_cache = {}
    # def kill_hanged_connection(self):
    #     dead_sql = []
    #
    def get_table_name(self,table_name):
        if '.' in table_name:
            return table_name
        else:
            return self.t_data['database']+'.'+table_name

    def keep_connect(self):
        can_update_sql = []
        for sql in self.sql_pool.values():
            if int(time.time()) - sql.get_last_connect_time() > 30:
                print(id(sql),'超时了')
                can_update_sql.append(sql)
        # count = 0
        # kill_connect = []
        # for sql in can_update_sql:
        #     if sql.is_busy() == False:
        #         kill_connect.append(id(sql))
        #         count += 1
        self.kill_unused_connection(can_update_sql)
        return

    def kill_unused_connection(self, kill_connect):
        count = 0
        for sql_id in kill_connect:
            if sql_id in self.sql_pool:
                if self.sql_pool[sql_id].is_busy() is False:
                    self.sql_pool.pop(sql_id)
                    count += 1
        print('计划清理',len(kill_connect),'个空闲连接，实际清理',count,'个')
        return

    def get_table_data(self,table,query):
        try:
            return self.mem_cache[table][query]
        except:
            return None

    def clean_table(self,table):
        try:
            return self.mem_cache.pop(table) if table in self.mem_cache else None
        except:
            return None

    def find_free_sql(self):
        for sql in self.sql_pool.values():
            if sql.is_busy() is False:
                sql.become_busy()
                return sql
        print('数据库连接池全忙状态，创建新的数据库链接')
        sql = self.create_new_sql()
        sql.become_busy()
        self.add_new_sql(sql)
        # print('创建完毕')
        # dbg_db('数据库连接池全忙状态，创建新的数据库链接', '新连接id:', id(sql))
        return sql

    def add_new_sql(self, sql):
        self.sql_pool[id(sql)] = sql
        return

    def new(self):
        sql = self.create_new_sql()
        # print(id(sql))
        self.add_new_sql(sql)
        return

    def create_new_sql(self, ):
        sql = Base(self.t_data['host'], self.t_data['user'], self.t_data['password'], self.t_data['database'],self.t_data['engine'])
        # sql.become_busy()
        # self.sql_pool.append(sql)
        return sql

    def create(self, table, colums, table_comment = '',show_sql=False):
        sql = self.find_free_sql()
        # sql.become_busy()
        print('执行这次sql请求的链接是', id(sql))
        result = sql.create(table, colums, table_comment,show_sql)
        sql.become_free()
        return result

    def insert(self, table, params, show_sql=False):
        sql = self.find_free_sql()
        table = self.get_table_name(table)
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.insert(table, params, show_sql)
        sql.become_free()
        self.clean_table(table)
        return result

    def find(self, table, conditions, or_cond, fields=('*',), order=None, show_sql=False,from_cache = False):
        sql = self.find_free_sql()
        table = self.get_table_name(table)
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        if from_cache == True:
            query = sql.find_info(table, conditions, or_cond, fields, None, order, None)
            query += " limit 1"
            res = self.get_table_data(table,query)
            if res is not None:
                sql.become_free()
                return res

        result = sql.find(table, conditions, or_cond, fields, order, show_sql)
        if result is None:
            return
        self.mem_cache[table] = {
            result['query']: result['result']
        }
        sql.become_free()
        return result['result']

    def select(self, table, conditions, or_cond, fields=('*',), group=None, order=None, limit=None, show_sql=False,from_cache=False):
        sql = self.find_free_sql()
        table = self.get_table_name(table)
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        if from_cache == True:
            query = sql.find_info(table, conditions, or_cond, fields, group, order, limit)
            res = self.get_table_data(table, query)
            if res is not None:
                sql.become_free()
                return res

        result = sql.select(table, conditions, or_cond, fields, group, order, limit, show_sql)
        if result is None:
            return
        self.mem_cache[table] = {
            result['query']:result['result']
        }
        sql.become_free()
        return result['result']

    def update(self, table, conditions,or_cond, params, show_sql=False):
        sql = self.find_free_sql()
        table = self.get_table_name(table)
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        # print('这次更新的params是',params)
        result = sql.update(table, conditions, or_cond, params, show_sql)
        self.clean_table(table)
        sql.become_free()
        return result

    def delete(self, table, conditions, or_cond, show_sql=False):
        sql = self.find_free_sql()
        table = self.get_table_name(table)
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.delete(table, conditions, or_cond,  show_sql)
        self.clean_table(table)
        sql.become_free()
        return result

    def query(self, sql_query, show_sql=False):
        # print(sql)
        sql = self.find_free_sql()
        # sql.become_busy()
        print('执行这次sql请求的链接是', id(sql))
        result = sql.query_one(sql_query, show_sql)
        sql.become_free()
        return result

    def truncate(self, table, show_sql=False):
        sql = self.find_free_sql()
        # sql.become_busy()
        # print('执行这次sql请求的链接是', id(sql))
        result = sql.truncate(table, show_sql)
        sql.become_free()
        return result



import time
import pymysql
import sqlite3

mysql = 'mysql'
sqlite = 'sqlite'

ENGINE_DICT = {
    'mysql':pymysql,
    'sqlite':sqlite3
}

class Base(object):

    def __init__(self, host, user, psw, dbname,engine):
        self.id = id(self)
        self._host = host
        self._user = user
        self._psw = psw
        self._dbname = dbname
        self._engine = engine
        self.db_engine = None
        if self._engine in ENGINE_DICT:
            self.db_engine = ENGINE_DICT[self._engine]
        # 选择数据库引擎
        self._is_busy = 0
        self._tables = {}
        self.init_time = int(time.time())
        self.executing_query = ''
        self.last_execute_time = int(time.time())
        self.db = None
        self.connect_db()
        self.last_connect_time = int(time.time())
        # print(self.id)

    def connect_db(self):
        try:
            self.db = self.connect()
            if self._engine != sqlite:
                self._load_tables()
            # print('数据库模块连接成功')
            print('connect success!')
            # IntervalTask(30, self.keep_connect)
        except Exception as e:
            # print(e)
            print('connect fail', str(e))
            # print('数据库模块连接成功',str(e))
            pass

    def connect(self):
        if self._engine == mysql:
            self.db = self.db_engine.connect(self._host, self._user, self._psw, self._dbname, charset='utf8',autocommit=True)
            return self.db
        if self._engine == sqlite:
            self.db = self.db_engine.connect(self._dbname)
            return self.db


    def become_busy(self):
        self._is_busy = 1
        return

    def become_free(self):
        self._is_busy = 0
        self.executing_query = ''
        return

    def is_busy(self):
        if self._is_busy == 1:
            return True
        else:
            return False

    def keep_connect(self):
        if self.db is None:
            self.connect_db()
        if self._engine != sqlite:
            self.query('select 1')



    def load_an_table(self, table):
        if self._engine == sqlite:
            sql = 'PRAGMA table_info(%s)'%table
        else:
            sql = 'show fields from ' + table
        res = self.query(sql, show_sql=False)
        if res is None:
            return
        column_list = list(map(lambda x: x[0], res))
        if 'signal' in column_list:
            column_list.remove('signal')
        self._tables[table] = column_list
        return column_list

    # 加载所有数据库表名
    def _load_tables(self, show_sql=False):
        sql = 'show tables'
        res = self.query(sql, show_sql)
        tables = tuple(map(lambda x: x[0], res))
        # print(tables)
        for table in tables:
            self.load_an_table(table)

    def _load_all_fileds(self):
        pass

    def create(self, table, colums, table_comment='',show_sql=False):
        sql = 'create table %s(' % table

        tail = ''

        for item in colums:
            col = ''
            if self._engine == sqlite:
                item= item[:-1]
            for i in item:
                col += str(i)
                col += ' '
            col += ','
            tail += col

        tail = tail[:-1] + ')'
        sql += tail
        # sql += 'charset=utf8mb4'
        if self._engine == mysql:
            sql += 'charset=utf8mb4,engine=innodb,comment="%s"'%(table_comment)
        if self._engine == sqlite:
            sql = sql.replace('int AUTO_INCREMENT primary key','INTEGER PRIMARY KEY AUTOINCREMENT')
        self.query(sql, show_sql)
        self.db.commit()
        return

    def find_info(self, table, conditions, or_cond, fields=None, group=None, order=None, limit=None):
        if len(fields) == 1 and fields[0] == '*':
            if table in self._tables:
                fields = self._tables[table]
            else:
                fields = self.load_an_table(table)
        if fields is None:
            return
        sql = 'select %s from %s where  ' % (','.join(fields), table)
        # if conditions == []:
        sql = Base.bind_conditions(sql, conditions, or_cond)

        if group is not None:
            sql += 'group by '
            for i in group:
                sql += '%s ,' % i
            sql = sql[:-1]

        if order is not None:
            sql += 'order by %s %s , id %s ' % (order[0], order[1], order[1])

        if limit is not None:
            sql += 'limit '
            for i in limit:
                sql += '%s, ' % i
            sql = sql[:-2]
        return sql

    @staticmethod
    def bind_conditions(sql, conditions, or_cond,unique = True):
        # print(sql)
        if conditions is None:
            conditions = []
        if or_cond is None:
            or_cond = []
        if len(or_cond) + len(conditions) == 0:
            return sql[:-7]

        if len(conditions) > 0:
            for unit in conditions:
                value = unit[2]
                if type("") == type(value):
                    value = "'%s'" % value
                if 'username' in unit[0]:
                    sql = sql + "binary %s %s %s " % (unit[0], unit[1], value) + "  and "
                else:
                    sql = sql + " %s %s %s " % (unit[0], unit[1], value) + "  and "
            sql = sql[:-4]
            if len(or_cond) > 0:
                sql += ' or '

        if len(or_cond) > 0:
            for unit in or_cond:
                value = unit[2]
                if type("") == type(value):
                    value = "'%s'" % value
                if 'username' in unit[0]:
                    sql = sql + "binary %s %s %s " % (unit[0], unit[1], value) + ' or '
                else:
                    sql = sql + " %s %s %s " % (unit[0], unit[1], value) + ' or '
            sql = sql[:-3]

        if unique == False:
            sql = sql.replace('binary', '')

        return sql


    # 查找数据（单条）
    def find(self, table, conditions, or_cond, fields=('*',), order=None, show_sql=False):
        sql = self.find_info(table, conditions, or_cond, fields, None, order, None)
        if sql is None:
            return

        sql += " limit 1"
        #
        # self.db.commit()
        res = self.query(sql, show_sql)
        if res is None:
            return None

        if 0 == len(res):
            return None

        if table not in self._tables:
            self.load_an_table(table)

        if fields[0] == '*' and len(fields) == 1:
            fieldList = self._tables[table]

        else:
            fieldList = fields

        # self.db.commit()
        result = {
            'table':table,
            'query':sql,
            'result':dict(zip(fieldList, res[0]))
        }
        return result

    # 查找数据
    def select(self, table, conditions, or_cond, fields=('*',), group=None, order=None, limit=None, show_sql=False):
        sql = self.find_info(table, conditions, or_cond, fields, group, order, limit)
        if sql is None:
            return
        #
        res = self.query(sql, show_sql)
        if res is None:
            return None

        if 0 == len(res):
            return None

        if table not in self._tables:
            self.load_an_table(table)

        if fields[0] == '*' and len(fields) == 1:
            fieldList = self._tables[table]
        else:
            fieldList = fields

        result = []
        for data in res:
            data = dict(zip(fieldList, data))
            result.append(data)
        res = {
            'table': table,
            'query': sql,
            'result': result
        }
        return res

    def insert(self, table, content, show_sql=False):
        params = content
        keys = str(tuple(params.keys()))
        keys = keys.replace("'", "")
        values = str(tuple(params.values()))
        if 1 == len(params):
            keys = keys[:-2] + ")"
            values = values[:-2] + ")"

        sql = 'insert into %s%s values %s ;' % (table, keys, values)
        #
        self.query(sql, show_sql)
        return

    def update(self, table, conditions, or_cond, params,show_sql=False):
        # print('开始执行')
        if params == {} or params is None:
            # print('没有params，结束执行')
            return
        sql = 'update %s set ' % table
        for param, value in params.items():
            if type("") == type(value):
                value = "'%s'" % value
            sql = sql + " %s = %s," % (param, value)

        sql = sql[:-1] + ' where  '
        sql = Base.bind_conditions(sql, conditions, or_cond)
        # print(sql)

        self.query(sql, show_sql)
            # print('自动提交完毕')
        return

    def delete(self, table, condition, or_cond, show_sql=False):
        sql = 'delete from %s where  ' % table
        sql = Base.bind_conditions(sql, condition, or_cond)
        #  #
        self.query(sql, show_sql)
        return

    def query(self, sql, show_sql=False):
        if self._engine == sqlite:
            sql = sql.replace('binary','')
        self.executing_query = sql
        if sql == 'select 1':
            show_sql = False
        if show_sql is True:
            print(sql)
        if self._engine != sqlite:
            try:
                self.db.ping(reconnect=True)
            except Exception as e:
                print(sql, 'db error', str(e), 'reconnecting...')
                self.connect()
        cursor = self.db.cursor()
        try:
            # res = time.time()
            self.update_last_execute_time()
            cursor.execute(sql)
            results = cursor.fetchall()
            if sql != 'select 1':
                self.update_last_connect_time()
            if self._engine == sqlite:
                self.db.commit()
        except Exception as e:
            print('<--------DBERROR-------->')
            print(sql)
            print('execute fail!', str(e))
            print('<--------DBERROR-------->')
            results = None

        # self.update_last_connect_time()
        # self.executing_query = ''
        # if self._engine == sqlite:
        #     self.db.commit()
        # print(results)
        # print(sql,'执行时间',time.time()-res)
        # self.executing_query = ''
        return results
        # results = cursor.fetchall()

    def update_last_connect_time(self):
        self.last_connect_time = int(time.time())
        # print(id(self),'执行',self.executing_query,'更新时间')
        return

    def update_last_execute_time(self):
        self.last_execute_time = int(time.time())
        return

    def get_last_connect_time(self):
        return self.last_connect_time

    def query_one(self, sql,show_sql=True):
        if self._engine != sqlite:
            try:
                self.db.ping(reconnect=True)
            except Exception as e:
                print(sql, 'db error', str(e), 'reconnecting...')
                self.connect()
        # cur.execute(sql)
        # db.commit()
        if show_sql:
            print(sql)
        cursor = self.db.cursor()
        cursor.execute(sql)
        results = cursor.fetchall()
        if self._engine == sqlite:
            self.db.commit()
        return results

    def truncate(self, table, show_sql=False):
        sql = 'TRUNCATE TABLE %s' % table
        self.query(sql, show_sql)
        return

    @property
    def tables(self):
        return self._tables
